@@ -142,8 +142,8 @@ fn extend_tokens_from_range(
 
     // compute original mapped token range
     let extended = {
-        let fst_expanded = sema.descend_into_macros(first_token.clone());
-        let lst_expanded = sema.descend_into_macros(last_token.clone());
+        let fst_expanded = sema.descend_into_macros_single(first_token.clone());
+        let lst_expanded = sema.descend_into_macros_single(last_token.clone());
         let mut lca =
             algo::least_common_ancestor(&fst_expanded.parent()?, &lst_expanded.parent()?)?;
         lca = shallowest_node(&lca);
@@ -155,7 +155,7 @@ fn extend_tokens_from_range(
 
     // Compute parent node range
     let validate = |token: &SyntaxToken| -> bool {
-        let expanded = sema.descend_into_macros(token.clone());
+        let expanded = sema.descend_into_macros_single(token.clone());
         let parent = match expanded.parent() {
             Some(it) => it,
             None => return false,
@@ -246,7 +246,7 @@ fn pick_best(l: SyntaxToken, r: SyntaxToken) -> SyntaxToken {
     fn priority(n: &SyntaxToken) -> usize {
         match n.kind() {
             WHITESPACE => 0,
-            IDENT | T![self] | T![super] | T![crate] | LIFETIME_IDENT => 2,
+            IDENT | T![self] | T![super] | T![crate] | T![Self] | LIFETIME_IDENT => 2,
             _ => 1,
         }
     }
@@ -328,7 +328,7 @@ mod tests {
     use super::*;
 
     fn do_check(before: &str, afters: &[&str]) {
-        let (analysis, position) = fixture::position(&before);
+        let (analysis, position) = fixture::position(before);
         let before = analysis.file_text(position.file_id).unwrap();
         let range = TextRange::empty(position.offset);
         let mut frange = FileRange { file_id: position.file_id, range };
